from __future__ import print_function
from sys import stdout
import argparse
import csv

from . import BarCodeReader, BarCode

def main():
    p = argparse.ArgumentParser()
    p.add_argument('-c','--csv', action='store_true')
    p.add_argument('--try-harder', action='store_true')
    p.add_argument('image', nargs='+')
    p.add_argument('-P','--classpath', help=argparse.SUPPRESS)
    p.add_argument('-J','--java', help=argparse.SUPPRESS)
    args = p.parse_args()

    bcr = BarCodeReader(args.classpath, args.java)

    if args.csv:
        wr = csv.writer(stdout)
        wr.writerow(('Filename','Format','Type','Raw','Parsed'))

    for fn in args.image:
        bc = bcr.decode(fn, try_harder=args.try_harder)
        if args.csv:
            wr.writerow((fn, bc.format, bc.type, bc.raw, bc.parsed) if bc else (fn, 'ERROR', None, None, None))
        else:
            print("%s\n%s" % (fn, '='*len(fn)))
            if bc is None:
                print("  ERROR: Failed to decode barcode.")
            else:
                print("  Decoded %s barcode in %s format." % (bc.type, bc.format))
                print("  Raw text:    %r" % bc.raw)
                print("  Parsed text: %r\n" % bc.parsed)



import logging
import zxing
import os

test_barcode_dir = os.path.join(os.path.dirname(__file__), 'barcodes')

test_barcodes = [
    ( 'QR_CODE-easy.png', 'This should be QR_CODE' ),
    ( 'CODE_128-easy.jpg', 'This should be CODE_128' ),
    ( 'PDF_417-easy.bmp', 'This should be PDF_417' ),
    ( 'AZTEC-easy.jpg', 'This should be AZTEC' ),
    ( 'CODE_128-fun-with-whitespace.png', '\n\r\t\r\r\r\n ' ),
    ( 'QR_CODE-screen_scraping_torture_test.png',
      '\n\\n¡Atención ☹! UTF-8 characters,\n embedded newline, &amp; trailing whitespace\t ' ),
]

def test_decoding():
    reader = zxing.BarCodeReader()
    for filename, expected in test_barcodes:
        path = os.path.join(test_barcode_dir, filename)
        logging.debug('Trying to parse {}, expecting {!r}.'.format(path, expected))
        dec = reader.decode(path)
        if dec.parsed != expected:
            raise AssertionError('Expected {!r} but got {!r}'.format(expected, dec.parsed))

def test_parsing():
    dec = zxing.BarCode.parse("""
file:default.png (format: FAKE_DATA, type: TEXT):
Raw result:
Élan|\tthe barcode is taking off
Parsed result:
Élan
\tthe barcode is taking off
Found 4 result points:
  Point 0: (24.0,18.0)
  Point 1: (21.0,196.0)
  Point 2: (201.0,198.0)
  Point 3: (205.23952,21.0)
""".encode())
    assert dec.format == 'FAKE_DATA'
    assert dec.type == 'TEXT'
    assert dec.raw == 'Élan|\tthe barcode is taking off'
    assert dec.parsed == 'Élan\n\tthe barcode is taking off'
    assert dec.points == [(24.0,18.0),(21.0,196.0),(201.0,198.0),(205.23952,21.0)]



########################################################################
#
#  zxing.py -- a quick and dirty wrapper for zxing for python
#
#  this allows you to send images and get back data from the ZXing
#  library:  http://code.google.com/p/zxing/
#

from __future__ import print_function
from enum import Enum

from .version import __version__
import subprocess as sp, re, os

class BarCodeReader(object):
    cls = "com.google.zxing.client.j2se.CommandLineRunner"

    def __init__(self, classpath=None, java=None):
        self.java = java or 'java'
        if classpath:
            self.classpath = classpath if isinstance(classpath, str) else ':'.join(classpath)
        elif "ZXING_CLASSPATH" in os.environ:
            self.classpath = os.environ.get("ZXING_CLASSPATH","")
        else:
            self.classpath = os.path.join(os.path.dirname(__file__), 'java', '*')

    def decode(self, filename, try_harder=False, possible_formats=None):
        possible_formats = (possible_formats,) if isinstance(possible_formats, str) else possible_formats

        cmd = [self.java, '-cp', self.classpath, self.cls, filename]
        if try_harder:
            cmd.append('--try_harder')
        if possible_formats:
            for pf in possible_formats:
                cmd += ['--possible_formats', pf ]

        p = sp.Popen(cmd, stdout=sp.PIPE, universal_newlines=False)
        stdout, stderr = p.communicate()

        if not p.returncode:
            return BarCode.parse(stdout)

class CLROutputBlock(Enum):
    UNKNOWN = 0
    RAW = 1
    PARSED = 2
    POINTS = 3

class BarCode(object):
    @classmethod
    def parse(cls, zxing_output):
        block = CLROutputBlock.UNKNOWN
        format = type = None
        raw = parsed = b''
        points = []

        for l in zxing_output.splitlines(True):
            if block==CLROutputBlock.UNKNOWN:
                if l.endswith(b': No barcode found\n'):
                    return None
                m = re.search(rb"format:\s*([^,]+),\s*type:\s*([^)]+)", l)
                if m:
                    format, type = m.group(1).decode(), m.group(2).decode()
                elif l.startswith(b"Raw result:"):
                    block = CLROutputBlock.RAW
            elif block==CLROutputBlock.RAW:
                if l.startswith(b"Parsed result:"):
                    block = CLROutputBlock.PARSED
                else:
                    raw += l
            elif block==CLROutputBlock.PARSED:
                if re.match(rb"Found\s+\d+\s+result\s+points?", l):
                    block = CLROutputBlock.POINTS
                else:
                    parsed += l
            elif block==CLROutputBlock.POINTS:
                m = re.match(rb"\s*Point\s*\d+:\s*\(([\d.]+),([\d.]+)\)", l)
                if m:
                    points.append((float(m.group(1)), float(m.group(2))))

        raw = raw[:-1].decode()
        parsed = parsed[:-1].decode()
        return cls(format, type, raw, parsed, points)

    def __init__(self, format, type, raw, parsed, points):
        self.raw = raw
        self.parsed = parsed
        self.format = format
        self.type = type
        self.points = points

    def __repr__(self):
        return '{}({!r}, {!r}, {!r}, {!r}, {!r})'.format(self.__class__.__name__, self.raw, self.parsed, self.format, self.type, self.points)



#!/usr/bin/env python3

from setuptools import setup
from urllib.request import urlretrieve
from urllib.error import URLError
from os import access, path, R_OK
import subprocess as sp
import sys, re

if not sys.version_info[0] == 3:
    sys.exit("Python 2.x is not supported; Python 3.x is required.")

########################################

# Based on this recipe, adapted for Python 3, Git 2.8.x, and PEP-440 version identifiers
#   http://blogs.nopcode.org/brainstorm/2013/05/20/pragmatic-python-versioning-via-setuptools-and-git-tags/
#   https://www.python.org/dev/peps/pep-0440/#version-scheme

# Fetch version from git tags, and write to version.py.
# Also, when git is not available (PyPi package), use stored version.py.
version_py = path.join(path.dirname(__file__), 'zxing', 'version.py')

try:
    version_git = sp.check_output(["git", "describe", "--tags", "--dirty=_dirty"]).strip().decode('ascii')
    final, dev, blob, dirty = re.match(r'v?((?:\d+\.)*\d+)(?:-(\d+)-(g[a-z0-9]+))?(_dirty)?', version_git).groups()
    version_pep = final+('.dev%s+%s'%(dev,blob) if dev else '')+(dirty if dirty else '')
except:
    d = {}
    with open(version_py, 'r') as fh:
        exec(fh.read(), d)
        version_pep = d['__version__']
else:
    with open(version_py, 'w') as fh:
        print("# Do not edit this file, versioning is governed by git tags", file=fh)
        print('__version__="%s"' % version_pep, file=fh)

########################################

def download_java_files():
    files = {'java/javase.jar': 'https://repo1.maven.org/maven2/com/google/zxing/javase/3.3.1/javase-3.3.1.jar',
             'java/core.jar': 'https://repo1.maven.org/maven2/com/google/zxing/core/3.3.1/core-3.3.1.jar',
             'java/jcommander.jar': 'https://repo1.maven.org/maven2/com/beust/jcommander/1.72/jcommander-1.72.jar'}

    for fn, url in files.items():
        p = path.join(path.dirname(__file__), 'zxing', fn)
        if access(p, R_OK):
            print("Already have %s." % p)
        else:
            print("Downloading %s from %s ..." % (p, url))
            try:
                urlretrieve(url, p)
            except URLError as e:
                raise SystemExit(*e.args)
    return list(files.keys())

setup(
    name='zxing',
    version=version_pep,
    description="wrapper for zebra crossing (zxing) barcode library",
    long_description="More information: https://github.com/dlenski/python-zxing",
    url="https://github.com/dlenski/python-zxing",
    author='Daniel Lenski',
    author_email='dlenski@gmail.com',
    packages = ['zxing'],
    package_data = {'zxing': download_java_files()},
    entry_points = {'console_scripts': ['zxing=zxing.__main__:main']},
    test_suite = 'nose.collector',
    license='LGPL v3 or later',
)
